'use strict';
// The module 'vscode' contains the VS Code extensibility API
// Import the module and reference it with the alias vscode in your code below
import * as vscode from 'vscode';
import { ConnectionProvider, ConnectionNode } from './connections'
const { spawn } = require('child_process');

var dbSession = null;

// this method is called when your extension is activated
// your extension is activated the very first time the command is executed
export function activate(context: vscode.ExtensionContext) {
    const connectionProvider = new ConnectionProvider();
    let myOutputChannel = vscode.window.createOutputChannel('SQL Query Results');

    vscode.window.registerTreeDataProvider('connections', connectionProvider);

    // Use the console to output diagnostic information (console.log) and errors (console.error)
    // This line of code will only be executed once when your extension is activated
    console.log('Congratulations, your extension "frog" is now active!');

    // The command has been defined in the package.json file
    // Now provide the implementation of the command with  registerCommand
    // The commandId parameter must match the command field in package.json
    let disposable = vscode.commands.registerCommand('extension.executeSnippet', () => {
        // The code you place here will be executed every time your command is executed

        let editor = vscode.window.activeTextEditor;

        if (!editor) return;

        let selection = editor.selection;

        if (selection.isEmpty) return;

        let text = editor.document.getText(selection);        

        if (!dbSession) {
            let extPath = vscode.extensions.getExtension("todie.frog").extensionPath;
            dbSession = spawn("java", ['-jar', extPath + '/frog-runner.jar', '172.19.23.18', '1521', 'aax2sm', 'aax2sm', 'aax2sm']);

            let outputBuffer = "";

            dbSession.stdout.on('data', (data) => {
                outputBuffer += data.toString();
                
                try {
                    let json = JSON.parse(outputBuffer);

                    let cd = json['columnDefinitions']

                    if (!json['rows']) {
                        if (!json['userErrors']) {
                            myOutputChannel.append("SQL executed successfully.\n");
                        } else {
                            myOutputChannel.append(JSON.stringify(json, null ,2));
                        }
                    } else {
                        for (let row of json["rows"]) {
                            myOutputChannel.append("--- ROW ---\n");
                            for (let [index, value] of row["columns"].entries()) {
                                myOutputChannel.append(cd[index]['name'] + "  " + value['value'] + "\n");
                            }
                        }
                    }

                    myOutputChannel.append("-- " + new Date() + " -------------------------------\n");
                    outputBuffer = "";
                } catch (e) {}
            });
              
            dbSession.stderr.on('data', (data) => {
                myOutputChannel.show();
                myOutputChannel.append(data.toString());
            });

            dbSession.on('close', (code) => {
                console.log(`child process exited with code ${code}`);
                dbSession = null;
            });
        }

        if (!text) {
            vscode.window.showInformationMessage('No text selected');
            return;
        }
        
        myOutputChannel.append("-- " + new Date() + " -------------------------------\n"); //Executing: '" + text + "'\n");
        dbSession.stdin.write(text + "\n--- END ---\n");
    });

    context.subscriptions.push(disposable);
}

// this method is called when your extension is deactivated
export function deactivate() {
    if (dbSession) dbSession.stdin.end();
}